package cn.rongcloud.im.ui.fragment;

import android.content.Context;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.TextUtils;
import android.text.method.LinkMovementMethod;
import android.text.style.ClickableSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;

import io.rong.eventbus.EventBus;
import io.rong.imkit.RongContext;
import io.rong.imkit.RongIM;
import io.rong.imkit.model.ProviderTag;
import io.rong.imkit.model.UIMessage;
import io.rong.imkit.userInfoCache.RongUserInfoManager;
import io.rong.imkit.widget.provider.RecallMessageItemProvider;
import io.rong.imlib.model.UserInfo;
import io.rong.message.RecallNotificationMessage;

@ProviderTag(messageContent = RecallNotificationMessage.class, showPortrait = false, showProgress = false, showWarning = false, centerInHorizontal = true,
        showSummaryWithName = false)
public class EditableRecallMessageItemProvider extends RecallMessageItemProvider {

    public static class ReeditEvent {
        private String msgText;

        public ReeditEvent(String msgText) {
            this.msgText = msgText;
        }

        public String getMsgText() {
            return msgText;
        }
    }

    private class ReeditClickableSpan extends ClickableSpan {

        private String msgText;

        ReeditClickableSpan(String msgText) {
            this.msgText = msgText;
        }

        @Override
        public void onClick(@NonNull View widget) {
            EventBus.getDefault().post(new ReeditEvent(msgText));
        }
    }

    @Override
    public void bindView(View v, int position, RecallNotificationMessage content, UIMessage message) {
        ViewHolder viewHolder = (ViewHolder) v.getTag();

        if (content != null && message != null) {
            String information;
            if (content.getOperatorId().equals(RongIM.getInstance().getCurrentUserId())) {
                if (content.isAdmin()) {
                    information = RongContext.getInstance().getString(io.rong.imkit.R.string.rc_admin_recalled_a_message);
                } else {
                    information = RongContext.getInstance().getString(io.rong.imkit.R.string.rc_you_recalled_a_message);
                }
            } else {
                UserInfo userInfo = RongUserInfoManager.getInstance().getUserInfo(content.getOperatorId());
                if (userInfo != null && userInfo.getName() != null) {
                    if (content.isAdmin()) {
                        information = RongContext.getInstance().getString(io.rong.imkit.R.string.rc_admin_recalled_message);
                    } else {
                        information = userInfo.getName() + RongContext.getInstance().getString(io.rong.imkit.R.string.rc_recalled_a_message);
                    }
                } else {
                    information = content.getOperatorId() + RongContext.getInstance().getString(io.rong.imkit.R.string.rc_recalled_a_message);
                }
            }

            String msgText = message.getExtra();
            SpannableStringBuilder ssb = new SpannableStringBuilder(information);
            if (!TextUtils.isEmpty(msgText)) {
                String reedit = "重新编辑";
                ssb.append(" ").append(reedit);
                ssb.setSpan(new ReeditClickableSpan(msgText),
                        ssb.length() - reedit.length(), ssb.length(),
                        Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            }
            viewHolder.contentTextView.setText(ssb);
        }

    }

    @Override
    public View newView(Context context, ViewGroup group) {
        View view = LayoutInflater.from(context).inflate(io.rong.imkit.R.layout.rc_item_information_notification_message, null);
        ViewHolder viewHolder = new ViewHolder();
        viewHolder.contentTextView = (TextView) view.findViewById(io.rong.imkit.R.id.rc_msg);
        viewHolder.contentTextView.setMovementMethod(LinkMovementMethod.getInstance());
        view.setTag(viewHolder);

        return view;
    }

    private static class ViewHolder {
        TextView contentTextView;
    }
}
